﻿/**
 *	@brief		Declares TPM response messages
 *	@details	This module contains message definitions for TPM1.2 and TPM2.0 response codes.
 *	@file		TpmResponseMessages.h
 *	@copyright	Copyright 2014 - 2017 Infineon Technologies AG ( www.infineon.com )
 *
 *	@copyright	All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once

// TPM2.0 response messages
#define TPM_RC_SUCCESS_MESSAGE				L"TPM_SUCCESS: Successful completion of the operation."

// TPM2.0 '0x100' "format zero" error code messages
#define TPM_RC_INITIALIZE_MESSAGE			L"TPM_RC_INITIALIZE: TPM not initialized."
#define TPM_RC_SEQUENCE_MESSAGE				L"TPM_RC_SEQUENCE: Improper use of a sequence handle."
#define TPM_RC_FAILURE_MESSAGE				L"TPM_RC_FAILURE: Commands not being accepted because of a TPM failure. NOTE: This may be returned by TPM2_GetTestResult() as the testResult parameter."
#define	TPM_RC_PRIVATE_MESSAGE				L"TPM_RC_PRIVATE: "
#define TPM_RC_HMAC_MESSAGE					L"TPM_RC_HMAC: "
#define TPM_RC_DISABLED_MESSAGE				L"TPM_RC_DISABLED: "
#define TPM_RC_EXCLUSIVE_MESSAGE			L"TPM_RC_EXCLUSIVE: Command failed because audit sequence required exclusivity."
#define TPM_RC_AUTH_TYPE_MESSAGE			L"TPM_RC_AUTH_TYPE: Authorization handle is not correct for command."
#define TPM_RC_AUTH_MISSING_MESSAGE			L"TPM_RC_AUTH_MISSING: Command requires an authorization session for handle and it is not present."
#define TPM_RC_POLICY_MESSAGE				L"TPM_RC_POLICY: Policy Failure In Math Operation or an invalid authPolicy value."
#define TPM_RC_PCR_MESSAGE					L"TPM_RC_PCR: PCR check fail."
#define TPM_RC_PCR_CHANGED_MESSAGE			L"TPM_RC_PCR_CHANGED: PCR have changed since checked."
#define TPM_RC_UPGRADE_MESSAGE				L"TPM_RC_UPGRADE: For all commands other than TPM2_FieldUpgradeData(), this code indicates that the TPM is in field upgrade mode; for TPM2_FieldUpgradeData(), this code indicates that the TPM is not in field upgrade mode."
#define TPM_RC_TOO_MANY_CONTEXTS_MESSAGE	L"TPM_RC_TOO_MANY_CONTEXTS: Context ID counter is at maximum."
#define TPM_RC_AUTH_UNAVAILABLE_MESSAGE		L"TPM_RC_AUTH_UNAVAILABLE: AuthValue or authPolicy is not available for selected entity."
#define TPM_RC_REBOOT_MESSAGE				L"TPM_RC_REBOOT: A _TPM_Init and Startup(CLEAR) is required before the TPM can resume operation."
#define TPM_RC_UNBALANCED_MESSAGE			L"TPM_RC_UNBALANCED: The protection algorithms (hash and symmetric) are not reasonably balanced. The digest size of the hash must be larger than the key size of the symmetric algorithm."
#define TPM_RC_COMMAND_SIZE_MESSAGE			L"TPM_RC_COMMAND_SIZE: Command commandSize value is inconsistent with contents of the command buffer; either the size is not the same as the octets loaded by the hardware interface layer or the value is not large enough to hold a command header."
#define TPM_RC_COMMAND_CODE_MESSAGE			L"TPM_RC_COMMAND_CODE: Command code not supported."
#define TPM_RC_AUTHSIZE_MESSAGE				L"TPM_RC_AUTHSIZE: The value of authorizationSize is out of range or the number of octets in the Authorization Area is greater than required."
#define TPM_RC_AUTH_CONTEXT_MESSAGE			L"TPM_RC_AUTH_CONTEXT: Use of an authorization session with a context command."
#define TPM_RC_NV_RANGE_MESSAGE				L"TPM_RC_NV_RANGE: NV offset+size is out of range."
#define TPM_RC_NV_SIZE_MESSAGE				L"TPM_RC_NV_SIZE: Requested allocation size is larger than allowed."
#define TPM_RC_NV_LOCKED_MESSAGE			L"TPM_RC_NV_LOCKED: NV access locked."
#define TPM_RC_NV_AUTHORIZATION_MESSAGE		L"TPM_RC_NV_AUTHORIZATION: NV access authorization fails in command actions (this failure does not affect lockout action)."
#define TPM_RC_NV_UNINITIALIZED_MESSAGE		L"TPM_RC_NV_UNINITIALIZED: An NV Index is used before being initialized or the state saved by TPM2_Shutdown(STATE) could not be restored."
#define TPM_RC_NV_SPACE_MESSAGE				L"TPM_RC_NV_SPACE: Insufficient space for NV allocation."
#define TPM_RC_NV_DEFINED_MESSAGE			L"TPM_RC_NV_DEFINED: NV Index or persistent object already defined."
#define TPM_RC_BAD_CONTEXT_MESSAGE			L"TPM_RC_BAD_CONTEXT: Context in TPM2_ContextLoad() is not valid."
#define TPM_RC_CPHASH_MESSAGE				L"TPM_RC_CPHASH: CpHash value already set or not correct for use."
#define TPM_RC_PARENT_MESSAGE				L"TPM_RC_PARENT: Handle for parent is not a valid parent."
#define TPM_RC_NEEDS_TEST_MESSAGE			L"TPM_RC_NEEDS_TEST: Some function needs testing."
#define TPM_RC_NO_RESULT_MESSAGE			L"TPM_RC_NO_RESULT: Returned when an internal function cannot processa request due to an unspecified problem. This code is usually related to invalid parameters that are not properly filtered by the input unmarshalling code."
#define TPM_RC_SENSITIVE_MESSAGE			L"TPM_RC_SENSITIVE: The sensitive area did not unmarshal correctly after decryption - this code is used in lieu of the other unmarshalling errors so that an attacker cannot determine where the unmarshalling error occurred."
#define RC_MAX_FM0_MESSAGE					L"RC_MAX_FM0: Largest version 1 code that is not a warning."

// TPM2.0 '0x080' "format one" error code messages
#define TPM_RC_ASYMMETRIC_MESSAGE			L"TPM_RC_ASYMMETRIC: Asymmetric algorithm not supported or not correct."
#define TPM_RC_ATTRIBUTES_MESSAGE			L"TPM_RC_ATTRIBUTES: Inconsistent attributes."
#define TPM_RC_HASH_MESSAGE					L"TPM_RC_HASH: Hash algorithm not supported or not appropriate."
#define TPM_RC_VALUE_MESSAGE				L"TPM_RC_VALUE: Value is out of range or is not correct for the context."
#define TPM_RC_HIERARCHY_MESSAGE			L"TPM_RC_HIERARCHY: Hierarchy is not enabled or is not correct for the use."
#define TPM_RC_KEY_SIZE_MESSAGE				L"TPM_RC_KEY_SIZE: Key size is not supported."
#define TPM_RC_MGF_MESSAGE					L"TPM_RC_MGF: Mask generation function not supported."
#define TPM_RC_MODE_MESSAGE					L"TPM_RC_MODE: Mode of operation not supported."
#define TPM_RC_TYPE_MESSAGE					L"TPM_RC_TYPE: The type of the value is not appropriate for the use."
#define TPM_RC_HANDLE_MESSAGE				L"TPM_RC_HANDLE: The handle is not correct for the use."
#define TPM_RC_KDF_MESSAGE					L"TPM_RC_KDF: Unsupported key derivation function or function not appropriate for use."
#define TPM_RC_RANGE_MESSAGE				L"TPM_RC_RANGE: Value was out of allowed range."
#define TPM_RC_AUTH_FAIL_MESSAGE			L"TPM_RC_AUTH_FAIL: The authorization HMAC check failed and DA counter incremented."
#define TPM_RC_NONCE_MESSAGE				L"TPM_RC_NONCE: Invalid nonce size."
#define TPM_RC_PP_MESSAGE					L"TPM_RC_PP: Authorization requires assertion of PP."
#define TPM_RC_SCHEME_MESSAGE				L"TPM_RC_SCHEME: Unsupported or incompatible scheme."
#define TPM_RC_SIZE_MESSAGE					L"TPM_RC_SIZE: Structure is the wrong size."
#define TPM_RC_SYMMETRIC_MESSAGE			L"TPM_RC_SYMMETRIC: Unsupported symmetric algorithm or key size, or not appropriate for instance."
#define TPM_RC_TAG_MESSAGE					L"TPM_RC_TAG: Incorrect structure tag."
#define TPM_RC_SELECTOR_MESSAGE				L"TPM_RC_SELECTOR: Union selector is incorrect."
#define TPM_RC_INSUFFICIENT_MESSAGE			L"TPM_RC_INSUFFICIENT: The TPM was unable to unmarshal a value because there were not enough octets in the input buffer."
#define TPM_RC_SIGNATURE_MESSAGE			L"TPM_RC_SIGNATURE: The signature is not valid."
#define TPM_RC_KEY_MESSAGE					L"TPM_RC_KEY: Key fields are not compatible with the selected use."
#define TPM_RC_POLICY_FAIL_MESSAGE			L"TPM_RC_POLICY_FAIL: A policy check failed."
#define TPM_RC_INTEGRITY_MESSAGE			L"TPM_RC_INTEGRITY: Integrity check failed."
#define TPM_RC_TICKET_MESSAGE				L"TPM_RC_TICKET: Invalid ticket."
#define TPM_RC_RESERVED_BITS_MESSAGE		L"TPM_RC_RESERVED_BITS: Reserved bits not set to zero as required."
#define TPM_RC_BAD_AUTH_MESSAGE				L"TPM_RC_BAD_AUTH: Authorization failure without DA implications."
#define TPM_RC_EXPIRED_MESSAGE				L"TPM_RC_EXPIRED: The policy has expired."
#define TPM_RC_POLICY_CC_MESSAGE			L"TPM_RC_POLICY_CC: The commandCode in the policy is not the commandCode of the command or the command code in a policy command references a command that is not implemented."
#define TPM_RC_BINDING_MESSAGE				L"TPM_RC_BINDING: Public and sensitive portions of an object are not cryptographically bound."
#define TPM_RC_CURVE_MESSAGE				L"TPM_RC_CURVE: Curve not supported."
#define TPM_RC_ECC_POINT_MESSAGE			L"TPM_RC_ECC_POINT: Point is not on the required curve."

// TPM2.0 '0x900' "format zero" warning messages
#define TPM_RC_CONTEXT_GAP_MESSAGE			L"TPM_RC_CONTEXT_GAP: Gap for context ID is too large."
#define TPM_RC_OBJECT_MEMORY_MESSAGE		L"TPM_RC_OBJECT_MEMORY: Out of memory for object contexts."
#define TPM_RC_SESSION_MEMORY_MESSAGE		L"TPM_RC_SESSION_MEMORY: Out of memory for session contexts."
#define TPM_RC_MEMORY_MESSAGE				L"TPM_RC_MEMORY: Out of shared object/session memory or need space for internal operations."
#define TPM_RC_SESSION_HANDLES_MESSAGE		L"TPM_RC_SESSION_HANDLES: Out of session handles - a session must be flushed before a new session may be created."
#define TPM_RC_OBJECT_HANDLES_MESSAGE		L"TPM_RC_OBJECT_HANDLES: Out of object handles - the handle space for objects is depleted and a reboot is required NOTE This cannot occur on the reference implementation. NOTE There is no reason why an implementation would implement a design that would deplete handle space. Platformspecifications are encouraged to forbid it."
#define TPM_RC_LOCALITY_MESSAGE				L"TPM_RC_LOCALITY: Bad locality."
#define TPM_RC_YIELDED_MESSAGE				L"TPM_RC_YIELDED: The TPM has suspended operation on the command; forward progress was made and the command may be retried. See Part 1, 'Multi-tasking.'NOTE This cannot occur on the reference implementation."
#define TPM_RC_CANCELED_MESSAGE				L"TPM_RC_CANCELED: The command was canceled."
#define TPM_RC_TESTING_MESSAGE				L"TPM_RC_TESTING: TPM is performing self-tests."
#define TPM_RC_REFERENCE_H0_MESSAGE			L"TPM_RC_REFERENCE_H0: The 1st handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H1_MESSAGE			L"TPM_RC_REFERENCE_H1: The 2nd handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H2_MESSAGE			L"TPM_RC_REFERENCE_H2: The 3rd handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H3_MESSAGE			L"TPM_RC_REFERENCE_H3: The 4th handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H4_MESSAGE			L"TPM_RC_REFERENCE_H4: The 5th handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H5_MESSAGE			L"TPM_RC_REFERENCE_H5: The 6th handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_H6_MESSAGE			L"TPM_RC_REFERENCE_H6: The 7th handle in the handle area references atransient object or session that is not loaded."
#define TPM_RC_REFERENCE_S0_MESSAGE			L"TPM_RC_REFERENCE_S0: The 1st authorization session handle references a session that is not loaded."
#define TPM_RC_REFERENCE_S1_MESSAGE			L"TPM_RC_REFERENCE_S1: The 2nd authorization session handle references a session that is not loaded."
#define TPM_RC_REFERENCE_S2_MESSAGE			L"TPM_RC_REFERENCE_S2: The 3rd authorization session handle references a session that is not loaded."
#define TPM_RC_REFERENCE_S3_MESSAGE			L"TPM_RC_REFERENCE_S3: The 4th authorization session handle."
#define TPM_RC_REFERENCE_S4_MESSAGE			L"TPM_RC_REFERENCE_S4: The 5th session handle references a session that is not loaded."
#define TPM_RC_REFERENCE_S5_MESSAGE			L"TPM_RC_REFERENCE_S5: The 6th session handle references a session that is not loaded."
#define TPM_RC_REFERENCE_S6_MESSAGE			L"TPM_RC_REFERENCE_S6: The 7th authorization session handle references a session that is not loaded."
#define TPM_RC_NV_RATE_MESSAGE				L"TPM_RC_NV_RATE: The TPM is rate-limiting accesses to prevent wear out of NV."
#define TPM_RC_LOCKOUT_MESSAGE				L"TPM_RC_LOCKOUT: Authorizations for objects subject to DAprotection are not allowed at this time because the TPM is in DA lockout mode."
#define TPM_RC_RETRY_MESSAGE				L"TPM_RC_RETRY: The TPM was not able to start the command."
#define TPM_RC_NV_UNAVAILABLE_MESSAGE		L"TPM_RC_NV_UNAVAILABLE: Tthe command may require writing of NV and NV is not current accessible."
#define TPM_RC_NOT_USED_MESSAGE				L"TPM_RC_NOT_USED: This value is reserved and shall not be returned by the TPM."

// TPM1.2 response messages
#define TPM_SUCCESS_MESSAGE					L"TPM_SUCCESS: Successful completion of the operation."
#define TPM_AUTHFAIL_MESSAGE				L"TPM_AUTHFAIL: Authentication failed."
#define TPM_BADINDEX_MESSAGE				L"TPM_BADINDEX: The index to a PCR, DIR or other register is incorrect."
#define TPM_BAD_PARAMETER_MESSAGE			L"TPM_BAD_PARAMETER: One or more parameter is bad."
#define TPM_AUDITFAILURE_MESSAGE			L"TPM_AUDITFAILURE: An operation completed successfully but the auditing of that operation failed."
#define TPM_CLEAR_DISABLED_MESSAGE			L"TPM_CLEAR_DISABLED: The clear disable flag is set and all clear operations now require physical access."
#define TPM_DEACTIVATED_MESSAGE				L"TPM_DEACTIVATED: The TPM is deactivated."
#define TPM_DISABLED_MESSAGE				L"TPM_DISABLED: The TPM is disabled."
#define TPM_DISABLED_CMD_MESSAGE			L"TPM_DISABLED_CMD: The target command has been disabled."
#define TPM_FAIL_MESSAGE					L"TPM_FAIL: The operation failed."
#define TPM_BAD_ORDINAL_MESSAGE				L"TPM_BAD_ORDINAL: The ordinal was unknown or inconsistent."
#define TPM_INSTALL_DISABLED_MESSAGE		L"TPM_INSTALL_DISABLED: The ability to install an owner is disabled."
#define TPM_INVALID_KEYHANDLE_MESSAGE		L"TPM_INVALID_KEYHANDLE: The key handle can not be interpreted."
#define TPM_KEYNOTFOUND_MESSAGE				L"TPM_KEYNOTFOUND: The key handle points to an invalid key."
#define TPM_INAPPROPRIATE_ENC_MESSAGE		L"TPM_INAPPROPRIATE_ENC: Unacceptable encryption scheme."
#define TPM_MIGRATEFAIL_MESSAGE				L"TPM_MIGRATEFAIL: Migration authorization failed."
#define TPM_INVALID_PCR_INFO_MESSAGE		L"TPM_INVALID_PCR_INFO: PCR information could not be interpreted."
#define TPM_NOSPACE_MESSAGE					L"TPM_NOSPACE: No room to load key."
#define TPM_NOSRK_MESSAGE					L"TPM_NOSRK: There is no SRK set."
#define TPM_NOTSEALED_BLOB_MESSAGE			L"TPM_NOTSEALED_BLOB: An encrypted blob is invalid or was not created by this TPM."
#define TPM_OWNER_SET_MESSAGE				L"TPM_OWNER_SET: There is already an Owner."
#define TPM_RESOURCES_MESSAGE				L"TPM_RESOURCES: The TPM has insufficient internal resources to perform the requested action."
#define TPM_SHORTRANDOM_MESSAGE				L"TPM_SHORTRANDOM: A random string was too short."
#define TPM_SIZE_MESSAGE					L"TPM_SIZE: The TPM does not have the space to perform the operation."
#define TPM_WRONGPCRVAL_MESSAGE				L"TPM_WRONGPCRVAL: The named PCR value does not match the current PCR value."
#define TPM_BAD_PARAM_SIZE_MESSAGE			L"TPM_BAD_PARAM_SIZE: The paramSize argument to the command has the incorrect value."
#define TPM_SHA_THREAD_MESSAGE				L"TPM_SHA_THREAD: There is no existing SHA-1 thread."
#define TPM_SHA_ERROR_MESSAGE				L"TPM_SHA_ERROR: The calculation is unable to proceed because the existing SHA-1 thread has already encountered an error."
#define TPM_FAILEDSELFTEST_MESSAGE			L"TPM_FAILEDSELFTEST: Self-test has failed and the TPM has shutdown."
#define TPM_AUTH2FAIL_MESSAGE				L"TPM_AUTH2FAIL: The authorization for the second key in a 2 key function failed authorization."
#define TPM_BADTAG_MESSAGE					L"TPM_BADTAG: The tag value sent to for a command is invalid."
#define TPM_IOERROR_MESSAGE					L"TPM_IOERROR: An IO error occurred transmitting information to the TPM."
#define TPM_ENCRYPT_ERROR_MESSAGE			L"TPM_ENCRYPT_ERROR: The encryption process had a problem."
#define TPM_DECRYPT_ERROR_MESSAGE			L"TPM_DECRYPT_ERROR: The decryption process did not complete."
#define TPM_INVALID_AUTHHANDLE_MESSAGE		L"TPM_INVALID_AUTHHANDLE: An invalid handle was used."
#define TPM_NO_ENDORSEMENT_MESSAGE			L"TPM_NO_ENDORSEMENT: The TPM does not a EK installed."
#define TPM_INVALID_KEYUSAGE_MESSAGE		L"TPM_INVALID_KEYUSAGE: The usage of a key is not allowed."
#define TPM_WRONG_ENTITYTYPE_MESSAGE		L"TPM_WRONG_ENTITYTYPE: The submitted entity type is not allowed."
#define TPM_INVALID_POSTINIT_MESSAGE		L"TPM_INVALID_POSTINIT: The command was received in the wrong sequence relative to TPM_Init and a subsequent TPM_Startup."
#define TPM_INAPPROPRIATE_SIG_MESSAGE		L"TPM_INAPPROPRIATE_SIG: Signed data cannot include additional DER information."
#define TPM_BAD_KEY_PROPERTY_MESSAGE		L"TPM_BAD_KEY_PROPERTY: The key properties in TPM_KEY_PARMs are not supported by this TPM."
#define TPM_BAD_MIGRATION_MESSAGE			L"TPM_BAD_MIGRATION: The migration properties of this key are incorrect."
#define TPM_BAD_SCHEME_MESSAGE				L"TPM_BAD_SCHEME: The signature or encryption scheme for this key is incorrect or not permitted in this situation."
#define TPM_BAD_DATASIZE_MESSAGE			L"TPM_BAD_DATASIZE: The size of the data (or blob) parameter is bad or inconsistent with the referenced key."
#define TPM_BAD_MODE_MESSAGE				L"TPM_BAD_MODE: A mode parameter is bad, such as capArea or subCapArea for TPM_GetCapability, physicalPresence parameter for TPM_PhysicalPresence, or migrationType for TPM_CreateMigrationBlob."
#define TPM_BAD_PRESENCE_MESSAGE			L"TPM_BAD_PRESENCE: Either the physicalPresence or physicalPresenceLock bits have the wrong value."
#define TPM_BAD_VERSION_MESSAGE				L"TPM_BAD_VERSION: The TPM cannot perform this version of the capability."
#define TPM_NO_WRAP_TRANSPORT_MESSAGE		L"TPM_NO_WRAP_TRANSPORT: The TPM does not allow for wrapped transport sessions."
#define TPM_AUDITFAIL_UNSUCCESSFUL_MESSAGE	L"TPM_AUDITFAIL_UNSUCCESSFUL: TPM audit construction failed and the underlying command was returning a failure code also."
#define TPM_AUDITFAIL_SUCCESSFUL_MESSAGE	L"TPM_AUDITFAIL_SUCCESSFUL: TPM audit construction failed and the underlying command was returning success."
#define TPM_NOTRESETABLE_MESSAGE			L"TPM_NOTRESETABLE: Attempt to reset a PCR register that does not have the resettable attribute."
#define TPM_NOTLOCAL_MESSAGE				L"TPM_NOTLOCAL: Attempt to reset a PCR register that requires locality and locality modifier not part of command transport."
#define TPM_BAD_TYPE_MESSAGE				L"TPM_BAD_TYPE: Make identity blob not properly typed."
#define TPM_INVALID_RESOURCE_MESSAGE		L"TPM_INVALID_RESOURCE: When saving context identified resource type does not match actual resource."
#define TPM_NOTFIPS_MESSAGE					L"TPM_NOTFIPS: The TPM is attempting to execute a command only available when in FIPS mode."
#define TPM_INVALID_FAMILY_MESSAGE			L"TPM_INVALID_FAMILY: The command is attempting to use an invalid family ID."
#define TPM_NO_NV_PERMISSION_MESSAGE		L"TPM_NO_NV_PERMISSION: The permission to manipulate the NV storage is not available."
#define TPM_REQUIRES_SIGN_MESSAGE			L"TPM_REQUIRES_SIGN: The operation requires a signed command."
#define TPM_KEY_NOTSUPPORTED_MESSAGE		L"TPM_KEY_NOTSUPPORTED: Wrong operation to load an NV key."
#define TPM_AUTH_CONFLICT_MESSAGE			L"TPM_AUTH_CONFLICT: NV_LoadKey blob requires both owner and blob authorization."
#define TPM_AREA_LOCKED_MESSAGE				L"TPM_AREA_LOCKED: The NV area is locked and not writable."
#define TPM_BAD_LOCALITY_MESSAGE			L"TPM_BAD_LOCALITY: The locality is incorrect for the attempted operation."
#define TPM_READ_ONLY_MESSAGE				L"TPM_READ_ONLY: The NV area is read only and can't be written to."
#define TPM_PER_NOWRITE_MESSAGE				L"TPM_PER_NOWRITE: There is no protection on the write to the NV area."
#define TPM_FAMILYCOUNT_MESSAGE				L"TPM_FAMILYCOUNT: The family count value does not match."
#define TPM_WRITE_LOCKED_MESSAGE			L"TPM_WRITE_LOCKED: The NV area has already been written to."
#define TPM_BAD_ATTRIBUTES_MESSAGE			L"TPM_BAD_ATTRIBUTES: The NV area attributes conflict."
#define TPM_INVALID_STRUCTURE_MESSAGE		L"TPM_INVALID_STRUCTURE: The structure tag and version are invalid or inconsistent."
#define TPM_KEY_OWNER_CONTROL_MESSAGE		L"TPM_KEY_OWNER_CONTROL: The key is under control of the TPM Owner and can only be evicted by the TPM Owner."
#define TPM_BAD_COUNTER_MESSAGE				L"TPM_BAD_COUNTER: The counter handle is incorrect."
#define TPM_NOT_FULLWRITE_MESSAGE			L"TPM_NOT_FULLWRITE: The write is not a complete write of the area."
#define TPM_CONTEXT_GAP_MESSAGE				L"TPM_CONTEXT_GAP: The gap between saved context counts is too large."
#define TPM_MAXNVWRITES_MESSAGE				L"TPM_MAXNVWRITES: The maximum number of NV writes without an owner has been exceeded."
#define TPM_NOOPERATOR_MESSAGE				L"TPM_NOOPERATOR: No operator AuthData value is set."
#define TPM_RESOURCEMISSING_MESSAGE			L"TPM_RESOURCEMISSING: The resource pointed to by context is not loaded."
#define TPM_DELEGATE_LOCK_MESSAGE			L"TPM_DELEGATE_LOCK: The delegate administration is locked."
#define TPM_DELEGATE_FAMILY_MESSAGE			L"TPM_DELEGATE_FAMILY: Attempt to manage a family other then the delegated family."
#define TPM_DELEGATE_ADMIN_MESSAGE			L"TPM_DELEGATE_ADMIN: Delegation table management not enabled."
#define TPM_TRANSPORT_NOTEXCLUSIVE_MESSAGE	L"TPM_TRANSPORT_NOTEXCLUSIVE: There was a command executed outside of an exclusive transport session."
#define TPM_OWNER_CONTROL_MESSAGE			L"TPM_OWNER_CONTROL: Attempt to context save a owner evict controlled key."
#define TPM_DAA_RESOURCES_MESSAGE			L"TPM_DAA_RESOURCES: The DAA command has no resources available to execute the command."
#define TPM_DAA_INPUT_DATA0_MESSAGE			L"TPM_DAA_INPUT_DATA0: The consistency check on DAA parameter inputData0 has failed."
#define TPM_DAA_INPUT_DATA1_MESSAGE			L"TPM_DAA_INPUT_DATA1: The consistency check on DAA parameter inputData1 has failed."
#define TPM_DAA_ISSUER_SETTINGS_MESSAGE		L"TPM_DAA_ISSUER_SETTINGS: The consistency check on DAA_issuerSettings has failed."
#define TPM_DAA_TPM_SETTINGS_MESSAGE		L"TPM_DAA_TPM_SETTINGS: The consistency check on DAA_tpmSpecific has failed."
#define TPM_DAA_STAGE_MESSAGE				L"TPM_DAA_STAGE: The atomic process indicated by the submitted DAA command is not the expected process."
#define TPM_DAA_ISSUER_VALIDITY_MESSAGE		L"TPM_DAA_ISSUER_VALIDITY: The issuer's validity check has detected an inconsistency."
#define TPM_DAA_WRONG_W_MESSAGE				L"TPM_DAA_WRONG_W: The consistency check on w has failed."
#define TPM_BAD_HANDLE_MESSAGE				L"TPM_BAD_HANDLE: The handle is incorrect."
#define TPM_BAD_DELEGATE_MESSAGE			L"TPM_BAD_DELEGATE: Delegation is not correct."
#define TPM_BADCONTEXT_MESSAGE				L"TPM_BADCONTEXT: The context blob is invalid."
#define TPM_TOOMANYCONTEXTS_MESSAGE			L"TPM_TOOMANYCONTEXTS: Too many contexts held by the TPM."
#define TPM_MA_TICKET_SIGNATURE_MESSAGE		L"TPM_MA_TICKET_SIGNATURE: Migration authority signature validation failure."
#define TPM_MA_DESTINATION_MESSAGE			L"TPM_MA_DESTINATION: Migration destination not authenticated."
#define TPM_MA_SOURCE_MESSAGE				L"TPM_MA_SOURCE: Migration source incorrect."
#define TPM_MA_AUTHORITY_MESSAGE			L"TPM_MA_AUTHORITY: Incorrect migration authority."
#define TPM_PERMANENTEK_MESSAGE				L"TPM_PERMANENTEK: Attempt to revoke the EK and the EK is not revocable."
#define TPM_BAD_SIGNATURE_MESSAGE			L"TPM_BAD_SIGNATURE: Bad signature of CMK ticket."
#define TPM_NOCONTEXTSPACE_MESSAGE			L"TPM_NOCONTEXTSPACE: There is no room in the context list for additional contexts."
#define TPM_RETRY_MESSAGE					L"TPM_RETRY: The TPM is too busy to respond to the command immediately, but the command could be resubmitted at a later time. The TPM MAY return TPM_RETRY for any command at any time."
#define TPM_NEEDS_SELFTEST_MESSAGE			L"TPM_NEEDS_SELFTEST: TPM_ContinueSelfTest has not been run."
#define TPM_DOING_SELFTEST_MESSAGE			L"TPM_DOING_SELFTEST: The TPM is currently executing the actions of TPM_ContinueSelfTest because the ordinal required resources that have not been tested."
#define TPM_DEFEND_LOCK_RUNNING_MESSAGE		L"TPM_DEFEND_LOCK_RUNNING: The TPM is defending against dictionary attacks and is in some time-out period."