# Usage of TPMFactoryUpd

This document describes the usage of TPMFactoryUpd binary to
manipulate the Infineon Trusted Platform Module firmware for versions
1.2 or lower. The binary is based on source code from Infineon and
contains patches from Google. The TPMFactoryUpd is designed to help mitigate against ROCA attacks.

See the [README](../README.md) for URLs and more information.

To update the TPM chip you need enter the BIOS, in this case SeaBIOS, 
so what had to be done was via SeaBIOS: clear, enable, activate the TPM forces as reboot). run tpm_takeownership then tpm_clear, then reboot. then run the update utility. not exactly user friendly =/

## TL,DR;

0. ```sudo apt-get install tpm-tools tpmfactoryupd```
1. Take ownership of and clear the TPM;
```
$ sudo tpm_takeownership
$ sudo tpm_clear
```
2. Download binary
3. Upgrade TPM
```
sudo TPMFactoryUpd -update tpm12-pp -firmware TPM12_4.40.119.0_to_TPM12_4.43.257.0.BIN -access-mode 1
```
## Caveat Emptor : Hic sunt leones

Installing the TPM firmware update requires a hardware reset of the
TPM chip. This will destroy data on the TPM chip, including any data
you may have put on the chip, like encryption keys. The loss of
encryption keys can mean that you cannot decrypt your disks! You can
lose data forever. Please back-up your data.

## Clear the TPM firmware

Before you can update the TPM chip, you need to clear, enable, and
activate the TPM in SeaBIOS. At Purism, we use
[SeaBIOS](https://www.seabios.org/SeaBIOS) as it is part of Coreboot,
and an open source implementation of a 16bit X86 BIOS.

You will have had to have run the previous commands tpm_takeownership
and tpm_clear for this to work.

## Diagnostics

In PureOS the executable ends up under /usr/bin/ once installed.
Running the executable this way;

`$ sudo /usr/bin/TPMFactoryUpd -info`

Which should produce;
```
  **********************************************************************
  *    Infineon Technologies AG   TPMFactoryUpd   Ver 01.01.2459.00    *
  **********************************************************************

       TPM information:
       ----------------
       Firmware valid                    :    Yes
       TPM family                        :    1.2
       TPM firmware version              :    4.40.119.0
       TPM enabled                       :    Yes
       TPM activated                     :    Yes
       TPM owner set                     :    Yes
       TPM deferred physical presence    :    No (Not settable)
       Remaining updates                 :    64
```
There are additional programs in PureOS which allow you to query the
Infineon TPM chip and to create and back-up keys. Those tools are;
tpm-tools, simple-tpm-pk11. Once installed you can run tpm_version
this way;
```
tpm_version
TPM 1.2 Version Info:
  Chip Version:        1.2.4.40
  Spec Level:          2
  Errata Revision:     3
  TPM Vendor ID:       IFX
  Vendor Specific data: abad1dea deadbeef 1337ffff ff
  TPM Version:         01010000
  Manufacturer Info:   49465800
```
You can also run a self-test or get the results from the latest self-test.
```
tpm_selftest -l info
TPM Test Results: 800001ff
tpm_selftest succeeded
```

## Resources
- https://wiki.archlinux.org/index.php/Trusted_Platform_Module
- https://blog.habets.se/2013/11/TPM-chip-protecting-SSH-keys---properly.html
- https://www.chromium.org/developers/design-documents/chaps-technical-design
- https://wiki.archlinux.org/index.php/Trusted_Platform_Module
- https://github.com/iavael/infineon-firmware-updater/blob/master/Common/ErrorCodes.h
